var _           =   require('underscore'),
    crypto      =   require('crypto'),
    utilities   =   require('../utilities/index'),
    config      =   {}; //require('./config');
    

/*//// Constructor --------------------------------------------------------------------------------------
function Crypto () {}*/


function hash (data, callback) {
    return crypto.createHash(data.cipher)
        .update(data.payload, data.encodingIn)
        .digest(data.encodingOut);
}

function getRandomBytesFunction (type) {
    return function( callback ){
        
        // Synchronous
        if ( _.isUndefined(callback) || !_.isFunction(callback) ||  callback === null ) return crypto.randomBytes(config.randomBytes[type].length).toString(config.randomBytes[type].encoding);
        
        // Asynchronous
        crypto.randomBytes (config.randomBytes[type].length, function (e, buf){
            if (e) return callback ( new utilities.error.srv (202) );
            return callback ( null, buf.toString ( config.randomBytes[type].encoding) )
        });
    };
}

function getHashFunction (type) {
    return function(data){        
        var digest = data.value;
        for (var i = 0; i < config.hashes[type].rounds; i++) {
            digest = hash ( _.extend ( { payload: data.salt + digest }, config.hashes[type] ));
        }
        return digest;
    };
}

function getCreateHashFunction (type) {
    return function( value, callback ){
        if ( _.isUndefined(callback) ||  callback === null ) {
            var salt = this['getRandom' + config.hashes[type].randomBytes]();
            return { hash: module.exports[ 'hash' + type ]( { value: value, salt: salt }), salt: salt };
        }
        else this['getRandom' + config.hashes[type].randomBytes]( function (e, salt) { // generate salt
            if (e) return callback( e );
            return callback ( null, { 
                hash: module.exports[ 'hash' + type ]( { value: value, salt: salt }), 
                salt: salt
            });
        });
    };
}

function getCreateHMACFunction (type) {
    var options = config.hmac[type];
    return function (data) {
        var payload = ""; //, time = (new Date (data.time)).getTime();
        //if (data.time && options.linkExpiry) data.time = ( new Date ( time ) ).toISOString();
        _.each (options.props, function(value){ payload += data[value]; });
        return crypto.createHmac(options.cipher, options.key).update(payload).digest(options.encoding);
    };
}



//// Public --------------------------------------------------------------------------------------

module.exports.init = function(options){
    if (options && _.isObject(options)) _.extend(config, options);

    // random bytes functions
    // inputs: (callback)
    // returns: (e, random bytes)

    _.each (config.randomBytes, function( options, name) {
        module.exports['getRandom' + name] = getRandomBytesFunction(name); //hashing
    });

    // hashing functions
    // inputs: ( { value: String, salt: String }, callback)
    // returns: (e, hashDigest)

    // hash creatiion functions
    // inputs: ( password , callback)
    // returns: (e, {hash: digest, salt: salt})
    _.each (config.hashes, function(options, name){
        module.exports['hash' + name] = getHashFunction(name); //hashing
        if (options.randomBytes) module.exports['createHash' + name] = getCreateHashFunction(name); //hashing
    });

    
    // hmac functions
    // inputs: (data)
    // returns: (hmac digest)

    _.each (config.hmac, function(options, name) {
        module.exports[name + 'HMAC'] = getCreateHMACFunction(name);
    });

};



    


//// Testing --------------------------------------------------------------------------------------

if (process.env.testing === "true") {
    module.exports.hash =  hash;
}