var mongoose 				= 	require ('mongoose'),
	db 						= 	mongoose.connection,
	underScore				= 	require ('underscore'),
	utilities   			=   require ('../utilities/index'),
	config					= 	{},
	env 					= 	process.env.NODE_ENV || 'development',
	alive 					= 	false,
	models 					= 	mongoose.models,
	maintenanceFunctions 	= 	{},
	maintenanceIntervals 	= 	{},
	chores 					= 	{
									clean: function () { 
										//console.log ('**** Maintenance **** cleanStorage'); 
									}
								};
	
	

//// Public --------------------------------------------------------------------------------------

module.exports.startAllMaintenance = function () {
	underScore.each (config.maintenance, function (period, name){
		maintenanceIntervals [ name.toLowerCase() ] = maintenanceFunctions [ 'start'+name ] ();
	});
}

module.exports.stopAllMaintenance = function () {
	underScore.each (config.maintenance, function (period, name){
		maintenanceFunctions [ 'stop'+name ] ();
	});
}

module.exports.init = function (options, callback) {
	
	if (options && underScore.isObject(options)) underScore.extend (config, options);

	if ( !db.readyState && db.readyState !== 1 && db.readyState !== 2 ) {
		
		var dbURI = config.devURI, dbOptions = config.devOptions;
		if 		(env === 'staging') 	{ dbURI = config.stgURI; dbOptions = config.stgOptions; }
		else if (env === 'production') 	{ dbURI = config.prodURI; dbOptions = config.prodOptions; }

		utilities.logger.info ('Connecting to: ' + dbURI);

		mongoose.connect ( dbURI, dbOptions );

		db.on 	('error', 			function ( err ) { 
			utilities.logger.error ('Error connecting to database.');
			utilities.logger.error (err);
			if (!alive) mongoose.disconnect (function () { return callback (true); });
		});
		db.on 	('connected', 		function ( ) { utilities.logger.info ('Connected to database.'); })
		db.on 	('disconnected', 	function ( ) { utilities.logger.info ('Disconnected from database.'); });
		db.once ('open', 			function ( ) { 
			utilities.logger.info ('Database successfully opened.');
			alive = true;
			var databaseReady = underScore.after(underScore.keys(config.models).length, function(){ return callback (null); });
			if ( underScore.isEmpty ( mongoose.models ) ) underScore.each ( config.models, function (value, name) { 
				mongoose.model ( name, value.schema ); 
				if (env !== 'production') mongoose.models[name].ensureIndexes(function (err) { if (err) utilities.logger.error(err); });
				databaseReady();
			});	
			else return callback (null);
		});	
	} else return callback (null);


	// Set each start chore function
	underScore.each (config.maintenance, function (period, name){
		maintenanceFunctions['start'+name] = function () { return setInterval ( chores [ name.toLowerCase() ], period); }
	});

	// Set each stop Chore functions
	underScore.each (config.maintenance, function (period, name){
		maintenanceFunctions['stop'+name] = function () { return clearInterval ( maintenanceIntervals [ name.toLowerCase() ] ); }
	});

	module.exports.startAllMaintenance();

}