var Schema 		= 	require('mongoose').Schema,
	underScore 	= 	require('underscore'),
	crypto 		= 	require('../../crypto/index'),
	utilities   =   require('../../utilities/index'),
	
	phoneTypes 	= 	['Home', 'Work', 'Mobile', 'Other'],
	locationTypes=  ['Home', 'Work', 'Other'];



var schemaOptions = { safe: true };

var schema = new Schema({
	_company: 		{ type: Schema.Types.ObjectId, ref: 'Company' },
	active: 		{ type: Boolean, default: true},
	auth: 			{
						local: { 
							hash: 	String,
							salt: 	String,
							reset: 	{ code: {type: String}, time: {type: Date, default: Date.now} } 
						},
						facebook: 	{ id: String, hash: String },
						twitter: 	{},
						google: 	{},
						openID: 	{}
					},
	avatar: 		{ url: String, fileType: String, bytes: Number },
	email: 			{ type: String, lowercase: true, trim: true, required: true },
	location: 		[ {
						locType: 	{ type: String, trim: true, enum: phoneTypes},
						address1: 	{ type: String, trim: true},
						address2: 	{ type: String, trim: true},
						city: 		{ type: String, trim: true},
						state: 		{ type: String, trim: true},
						postal: 	{ type: String, trim: true},
						country: 	{ type: String, trim: true}
					} ],
	name: 			{ type: String, trim: true },
	phone: 			[ { 
						phType: 	{ type: String, trim: true, enum: phoneTypes}, 
						areaCode: 	{ type: String, lowercase: true, trim: true },
						number: 	{ type: String, lowercase: true, trim: true },
						ext: 		{ type: String, lowercase: true, trim: true },
					} ],
	notifications: 	[ {  _id:false, platform: String, token: String, targetArn: String  } ],
	settings: 		{
						products : {
							shuflUser: { interests: [ String ] }
						}
						//powers: 	{ superAdmin: [ String ], admin: [ String ], user: [ String ] },
						//client: 	{ name: String, values: Schema.Types.Mixed }
						//api: 		{ prodKeys: [ { name: String, key: String } ], testKeys: [ { name: String, key: String } ] }
						//devices: 	[ devicesID ]

					},
	time: 			{
						created: 	{ type: Date, default: Date.now },
						deactivated: Date
					},
	username: 		{ type: String, trim: true },
	verified: 		Boolean
	
}, schemaOptions);

//Methods

schema.methods.setLocalPassword = function ( password, callback ) {
  	utilities.logger.info('cs_database/bin/person setLocalPassword');
  	var person = this;
    crypto.createHashPassword( password, function (e, data) {
        if (e) return callback(e);
        underScore.extend (person.auth.local, data );
        return callback (null, person);
    });
};

schema.methods.setFacebookToken = function ( callback ) {
  	utilities.logger.info('cs_database/bin/person setFacebookToken');
  	var person = this;
    crypto.getRandomFacebookToken( function (e, data) {
        if (e) return callback(e);
        //person.auth.facebook.hash = data
        underScore.extend (person.auth.facebook, {hash:data} );
        return callback (null, person);
    });
};

schema.methods.verifyLocalPassword = function ( password, callback ) {
  	utilities.logger.info('cs_database/bin/person verifyLocalPassword');
  	var person = this,
  		digest = crypto.hashPassword( { value:password, salt:person.auth.local.salt });
    if (callback) return callback (digest === person.auth.local.hash ? null :  new utilities.error.app(402)); //async
    return digest === person.auth.local.hash; //sync
};


exports.schema = schema;