var _ = 		require('underscore'),
	util =      require('util'),
	logger =    require('../logger/index'),
	config = 	{},

	AbstractError = function (errType, code, constr) {
		Error.captureStackTrace(this, constr || this)
		this.message = config.messages['EN'][errType][code] + ' (' + code + ')' || 'Error'
		this.code = code;
		logger.error (this.stack);
	};
	
util.inherits(AbstractError, Error)
AbstractError.prototype.name = 'Abstract Error'


var DatabaseError = function (code) { DatabaseError.super_.call(this, 'db', code, this.constructor) };
util.inherits(DatabaseError, AbstractError);
DatabaseError.prototype.name = 'Database Error';

var ServerError = function (code) { ServerError.super_.call(this, 'srv', code, this.constructor) };
util.inherits(ServerError, AbstractError);
ServerError.prototype.name = 'Server Error';

var ApplicationError = function (code) {
  this.message = config.messages['EN'].app[code] + ' (' + code + ')' || 'Error'
  this.code = code;
}

var AWSError = function (code) { AWSError.super_.call(this, 'aws', config.awsErrorTranslate[code] || 800, this.constructor) };
util.inherits(AWSError, AbstractError);
AWSError.prototype.name = 'AWS Error';

var PaypalError = function (code) { PaypalError.super_.call(this, 'ppl', code || 900, this.constructor) };
util.inherits(PaypalError, AbstractError);
PaypalError.prototype.name = 'Paypal Error';

module.exports = {
  db: 	DatabaseError,
  srv: 	ServerError,
  app: 	ApplicationError,
  aws: 	AWSError,
  ppl: 	PaypalError,
  init: function(options){
			if (options && _.isObject(options)) _.extend(config, options);
		}
}