var cluster                 =   require('cluster'),
    numCPUs                 =   require('os').cpus().length,
    
    _                       =   require('underscore'),
    mongoose                =   require('mongoose'),
    passport                =   require('passport'),
    AWS                     =   require('aws-sdk'),
    logger                  =   require('./logger'),
    worker                  =   require('./worker'),

    
    env                     =   process.env.NODE_ENV || 'development',
    timeouts                =   [];


//// Public --------------------------------------------------------------------------------------


if (cluster.isWorker || process.env.testScript === "true") {
    module.exports = _.omit (worker, 'init');
}

module.exports.mongoose =   mongoose;
module.exports.passport =   passport;
module.exports.AWS =        AWS;
module.exports.env =        env;


module.exports.init = function (options) {

    logger.init (options.logger);
    
    if (cluster.isMaster) { 

        // Fork workers.
        for (var i = 0; i < numCPUs; i++) { 
            logger.info  ("utlWebServer init - Forking a Worker");
            logger.info  ("utlWebServer init - Using AWS Access Key Id: " + process.env.AWS_ACCESS_KEY_ID);
            //logger.info  ("utlWebServer init - Using AWS Secret Key: " + process.env.AWS_SECRET_KEY);
            cluster.fork(); 
        }

        cluster.on('fork', function(worker) {
            timeouts[worker.id] = setTimeout ( function () { logger.warn('utlWebServer init Worker #' + worker.id +' is not listening yet. Something must be wrong.') }, 5000);
        });

        cluster.on('listening', function (worker, address) {
            clearTimeout(timeouts[worker.id]);
            logger.info('utlWebServer init Worker #' + worker.id + ' now listening on port ' + address.port);
        });

        cluster.on('exit', function(worker, code, signal) {
            logger.error('utlWebServer init Worker #' + worker.id + ' ( ' + worker.process.pid + ') died ('+code+')');
            if (timeouts[worker.id]) clearTimeout(timeouts[worker.id]);
            if (cluster.workers.length < numCPUs) cluster.fork();
        });

        cluster.on('disconnect', function(worker) { 
            logger.warn('Worker #' + worker.id + ' ( ' + worker.process.pid + ') has disconnected'); 
            if (timeouts[worker.id]) clearTimeout(timeouts[worker.id]);
            cluster.fork();
        });
    
    } else {
        worker.init(options);
    }    
};