var _                   =   require ('underscore'),
    passport            =   require ('passport'),
    cookie              =   require ('express/node_modules/cookie'),
    cookieParser        =   require ('cookie-parser'),
    formidable          =   require ('formidable'),
    fs                  =   require ('fs'),
    utilities           =   require ('../utilities/index'),
    validator           =   require ('../validator/index'),
    util                =   require ('util'),
    tmpFolder           =   __dirname + '/../.tmp/',
    io                  =   null,
    controllers         =   { },
    restSessionStore    =   null;


function validateAndSanitize (args, api, session, callback) {
    utilities.logger.info('framework/interfaces validateAndSanitize');
    //utilities.analytics.trackEvent ({ ec: "REST " + api.version, ea: api.section, el: api.command, ev: 1, ep: 'req.path' });

    // check for valid session if required
    if (!controllers[api.version][api.section].api[api.command].authBypass) {
        utilities.logger.info ('CHECKAUTH - ' + api.section + ' ' + api.command);
        if (!session || !session.passport || !session.passport.user) {
            utilities.logger.info ('NOSESSION - ' + api.section + ' ' + api.command);
            return callback( new utilities.error.app(405) );
        }
    }

    // check for valid arguments
    var requiredProperties =    controllers[api.version][api.section].api[api.command].args.required || [],
        optionalProperties =    controllers[api.version][api.section].api[api.command].args.optional || [],
        allProperties =         _.union (requiredProperties, optionalProperties),
        data =                  allProperties.length ? ( _.pick (args, allProperties) ) : {};

    if ( requiredProperties.length && validator.missingProperties ( data, requiredProperties ) ) return callback ( new utilities.error.app(350) );

    allProperties = _.intersection(_.keys(data), allProperties) || []; // filter allProperties for only ones in the argument list

    if (allProperties.length && !validator.isValid( data, allProperties )) return callback ( new utilities.error.app(351) );
    else validator.sanitize (data, function (cleanData) {
        utilities.logger.info ('VALIDATOR: ' + api.section + ' ' + api.command + ' has valid properties' );
        //cleanData.session = session;
        return callback (null, cleanData);
    });
};


function goValidateREST (args, api, req, res, returnFunc) {                  

    validateAndSanitize (args, api, req.session, function ( e, cleanData ) {
        if (e) return res.send( { error: e.code || -1 } );
        if (!cleanData.username && cleanData.email) { cleanData.username = cleanData.email; req.query.username = cleanData.email; }
        req.data = cleanData;
        
        return controllers[api.version][api.section][api.command] (req, res, function (e, result, user) {
            if (e) { 
                utilities.logger.info('REST ERROR: ' + api.section + ' ' + api.command + ' * error *: ' + util.inspect(e)); 
                return res.send( { error: e.code || -1 } );
            } else { 
                utilities.logger.info('REST: ' + api.section + ' ' + api.command + ' success.'); 
                if (user) {
                    req.session[passport._key] = {};
                    req.session[passport._key][passport._userProperty] = user;
                    /*restSessionStore.set(req.session.id, req.session, function(e, sessionInfo){ 
                        if (e) utl.logger.warn ('Error saving session data');
                        return res.send( _.extend ({ error: 0 }, result) );   
                    });*/
                    return res.send( _.extend ({ error: 0 }, result) );   
                } else return res.send( _.extend ({ error: 0 }, result) );             
            }
        });
    });
}

function handleMultipartRest (req, res, api) {
    var form = new formidable.IncomingForm();
    form.uploadDir = tmpFolder;
    form.parse(req, function(e, data, files) {                    
        utilities.logger.info ('Form Data: ' + JSON.stringify(data) );
        if (e) {
            utilities.logger.info('REST: ' + api.section + ' ' + api.command + ' * error *: ' + util.inspect(e));
            return res.send( { error: (new utilities.error.app(409)).code || -1 } );
        } else {                        
            var fileArray = [];
            _.each(files, function (file) { fileArray.push( { size: file.size, path:file.path, mimeType:file.type, type:file.type } ); });
            
            //multipart data is usually url encoded. Decode strings here.
            _.each (data, function (value, prop) { 
                if ( _.isString (value) ) {
                    try {
                        data[prop] = decodeURIComponent(value);     
                    }
                    catch (e) {
                        utilities.logger.error ("Decode Component Failed!");
                        utilities.logger.error (e);
                        utilities.logger.error (value);
                    }
                }
            });

            return goValidateREST (_.extend ({}, data, { files:fileArray }), api, req, res);
        }
    });
}

//// Public --------------------------------------------------------------------------------------

exports.initREST = function (server, options) {
    utilities.logger.info('framework/interfaces initREST');
    
    restSessionStore = options.session.store;

    var getRESTFunction = function (api) {
        return function ( req, res ) { 
            
            utilities.logger.info ('COOKIES: ' + JSON.stringify (req.cookies) );
            utilities.logger.info ('REST: ' + api.section + ' ' + api.command );
            utilities.logger.info ('Data: ' + JSON.stringify(req.method === 'POST' ? req.body : req.query) );
            
            if ( req.is('multipart/form-data') && controllers[api.version][api.section].api[api.command].fileUploadAllowed ) handleMultipartRest (req, res, api)
            else return goValidateREST (req.method === 'POST' ? req.body : req.query, api, req, res);
        };
    };

    // listen for each rest command
    _.each ( controllers, function(sections, version){
        _.each (sections, function(commands, section){
            _.each (commands.api, function(commandsDetail, command){
                _.each (commandsDetail.methods, function(method){server[method] ( '/'+version+'/'+section+'/'+command, getRESTFunction({version:version, section:section, command:command})); });
            });
        });
    });

    // load balancer health check
    //server.all ( '/healthCheck1123581220', function (req, res ){ res.sendStatus(200); });    
}

exports.initWebsockets = function ( serverInstance, options ) {
    utilities.logger.info('framework/interfaces initWebsockets');

    io = require('socket.io')(serverInstance, options.socketIO);

    var getWebsocketsFunction = function (api) {
        return function (data, fileData) {
            
            var socket = this,
                currentCommand = controllers[api.version][api.section].api[api.command],
                path = '/'+api.version+'/'+api.section+'/'+api.command,
                
                returnFunc = function(e, result, user){
                    
                    if (user) {
                        socket.request.session[passport._key] = {};
                        socket.request.session[passport._key][passport._userProperty] = user;
                        options.session.store.set(socket.request.session.id, socket.request.session, function(e, sessionInfo){ if (e) utl.logger.warn ('Error saving session data'); });
                    }
                    
                    if (e) {
                        utilities.logger.info('WEBSOCKETS: ' + path + ' * error *: ' + e);
                        socket.emit(path, { error: e.code || -1 } );
                    } else {
                        utilities.logger.info('WEBSOCKETS: ' + path + ' success.');
                        socket.emit(path, _.extend ( { error: 0 }, result) );
                    }
                },
                
                goValidate = function () {
                    validateAndSanitize (data, api, socket.request.session, function ( e, cleanData ) {
                        if (e) return socket.emit('/'+api.version+'/'+api.section+'/'+api.command, { error: e.code || -1 } );
                            if (!cleanData.username && cleanData.email) cleanData.username = cleanData.email;
                            var req = _.extend ( {}, {session: socket.request.session || {}, body: socket.request.body || {}, query:socket.request.query || {}, headers: socket.request.headers || {} }, {body:cleanData, query:cleanData, data: cleanData, url:'/socket.io'} );
                                res = {};
                            controllers[api.version][api.section][api.command] (req, res, returnFunc);
                    });
                };

            //utilities.logger.info('SOCKET SESSION: ' + JSON.stringify(socket.request.session));

            //if ( currentCommand.fileUploadAllowed && data.files && data.files.length && buffer && buffer.length) { 
            if ( currentCommand.fileUploadAllowed ) { 
                if ( !data.files || !data.files.length ) utilities.logger.info('WEBSOCKETS: files parameter not found or empty' );
                else {
                    if ( !fileData || !fileData.length) utilities.logger.info('WEBSOCKETS: fileData parameter not found or empty' );
                    else {
                        var afterFileUpload = _.after(data.files.length, goValidate);
                        _.each (data.files, function (file, index) {
                            var fileName = tmpFolder + Date.now() + '_' + file;
                            fs.open(fileName, 'a', 0755, function (e, fd) {
                                if (e) utilities.logger.warn('WEBSOCKETS: unable to write ' + fileName );
                                else {
                                    fs.write(fd, fileData[index], null, 'Binary', function(e, written, buff) {
                                        if (e) utilities.logger.warn('WEBSOCKETS: error writing ' + fileName );
                                        fs.close(fd, function() { 
                                            utilities.logger.info (fileName + ' saved successfully!'); 
                                            data.files [index] = { path: fileName, size: fs.statSync(fileName).size, mimeType: 'image/jpeg' };
                                            afterFileUpload();
                                        });
                                    });
                                }
                            });
                        });
                    }
                }
            } else goValidate();
        };
    };

    //io.of('/shufl')
    io.use(function(socket, next) {
        var cookieHeader = socket.request.headers.cookie, session = null, id = null;

        if (!cookieHeader) return next('No cookie transmitted.');
        cookieHeader = cookie.parse(cookieHeader);
        
        utilities.logger.info(cookieHeader); 

        if (cookieHeader[options.session.secret]) {
            id = cookieParser.signedCookie (cookieHeader[options.session.secret], options.session.secret);
            
            if (cookieHeader[options.session.secret] == id || id === false) {
                utilities.logger.warn('Cookie is invalid'); 
                return next('Cookie is invalid');
            }
            options.session.store.get(id, function(e, sessionInfo){
                if (sessionInfo) socket.request.session = sessionInfo;
                else socket.request.session = {};
                socket.request.session.id = id;
                utilities.logger.info('session id: ' + socket.request.session.id); 
                utilities.logger.info('session: ' + JSON.stringify(socket.request.session)); 
                return next();
            });
        } else return next('No cookie transmitted.');
    })
    .on('connection', function (socket) {
        utilities.logger.info('Connected');
        socket.on ('error',         function (e) { utilities.logger.error('Connection error'); utilities.logger.error(e); });
        socket.on ('terminate',     function () { socket.disconnect(); utilities.logger.info('Client terminated connection'); });
        socket.on ('disconnect',    function () { utilities.logger.warn('Disconnected'); });

        // custom functions
        _.each ( controllers, function(sections, version){
            _.each (sections, function(commands, section){
                _.each (commands.api, function(commandsDetail, command){
                    socket.on ( '/'+version+'/'+section+'/'+command, getWebsocketsFunction ( { version:version, section:section, command:command } ) );
                });
            });
        });
    });
}


exports.setControllers = function ( customControllers ) {
    if (customControllers && _.isObject(customControllers)) _.extend (controllers, customControllers);
};