var _                   =   require('underscore'),
    winston             =   require('winston'),
    cloudWatch          =   require('winston-cloudwatch'),
    slack               =   require('./bin/slack').Slack,
    fs                  =   require('fs'),

    config              =   {},

    timestampFunc       =   function () { return Date.now(); },
    
    noop                =   function (dummy){};


// remove default console logger
winston.remove(winston.transports.Console);


//// Public ---------------------------------------------------------------------------------------
exports.init = function (options)  { 
    if (options && _.isObject(options)) _.extend(config, options);
    winston.addColors(config.loggerLevels.colors);

    fs.closeSync ( fs.openSync(config.filenameAll, 'w') ); //truncate file


    if (process.env.NODE_ENV === 'development') {

        winston.add( winston.transports.Console, { level:'warn', colorize: 'true', timestamp: timestampFunc });
        winston.add( winston.transports.File, { filename: config.filenameAll, colorize: 'true', timestamp: timestampFunc });
        if (config.cloudWatchLogging) winston.add( cloudWatch, config.awsCWDevOptions);
        if (config.slack && config.slack.enabled && config.slack.dev) winston.add( slack, config.slack.dev || {});

    } else if (process.env.NODE_ENV === 'staging') {

        winston.add( winston.transports.Console, { level:'warn', colorize: 'true', timestamp: timestampFunc });
        winston.add( winston.transports.File, { filename: config.filenameAll, colorize: 'true', timestamp: timestampFunc });
        if (config.cloudWatchLogging)  winston.add( cloudWatch, config.awsCWStgOptions);
        if (config.slack && config.slack.enabled && config.slack.stg) winston.add( slack, config.slack.stg || {});

    } else if (process.env.NODE_ENV === 'production') {
        
        winston.add(winston.transports.Console, {level:'warn', colorize: 'true', timestamp: timestampFunc } );
        if (config.cloudWatchLogging)  winston.add( cloudWatch, config.awsCWProdOptions);
        if (config.slack && config.slack.enabled && config.slack.prod) winston.add( slack, config.slack.prod || {});
    }

};

exports.info = process.env.NODE_ENV === 'production' ? noop : winston.info;
exports.warn = winston.warn;
exports.error = winston.error;