/*!
* mailer-node
* Email Layer
* Copyright(c) 2013 CryptoMill Technologies Ltd. <info@cryptomill.com>
*/

var _               =   require('underscore'),
    mailer          =   require("nodemailer"),
    htmlToText      =   require('nodemailer-html-to-text').htmlToText,
    
    config          =   {},
    utilities       =   require('../utilities/index'),
    transports      =   {};




//// Public --------------------------------------------------------------------------------------

exports.init = function (options)  { 
    if (options && _.isObject(options)) _.extend(config, options);
    
    // Set up transports
    _.each (config.transports, function (transport, name){
        if (transport.enabled) {
            transports[name] = mailer.createTransport(transport.transportObject(transport.options));
            transports[name].use('compile', htmlToText());
        }
        else transports[name] = false;
    });

    // To turn on the mailer in development, comment the following line.
    if (process.env.NODE_ENV === 'development') {
        if (!config.enabled.dev) exports.send = function (dummy, callback) { return callback (null, {}); };    
    }

    if (process.env.NODE_ENV === 'staging') {
        if (!config.enabled.stg) exports.send = function (dummy, callback) { return callback (null, {}); };
    }

    if (process.env.NODE_ENV === 'production') {
        if (!config.enabled.prod) exports.send = function (dummy, callback) { return callback (null, {}); };    
    }

};

exports.send = function (data, callback) {
    
    utilities.logger.info('framework/mailer send'); 

    var mailType = config.types[data.mailType];
    if (!mailType) return callback (-1);
    
    if (!data.locale || !config.strings[data.locale]) data.locale = 'en';
    else data.locale = data.locale.toLowerCase();
    
    var templateArgs = mailType.setArgs(data);

    _.extend (templateArgs, data);

    var options = { 
            to:         data.to,
            cc:         data.cc || "",
            bcc:        data.bcc || "",
            subject:    mailType.template.subject(templateArgs),
            //text:       mailType.template.text(templateArgs),
            html:       mailType.template.html(templateArgs)
        },
        mailOptions = _.extend(config.defaults, options);

    transports.SES.sendMail ( mailOptions, function (e, response) {
        if (e) { utilities.logger.error(e); return callback ( new utilities.error.aws (e.code) ); }
        return callback(null);
    });
};