var underScore =    require('underscore'),
    AWS =           require('aws-sdk'),
    sns =           null,
    models =        require ('mongoose').models,    
    utilities =     require('../../utilities'),
    config =        {};



//// Public ---------------------------------------------------------------------------------------

module.exports.init = function (options) {
    utilities.logger.info('framework/notify/bin/awssns init');
    if (options) underScore.extend (config, options);
    AWS.config.update (underScore.pick(config, 'accessKeyId', 'secretAccessKey', 'region'));
    sns = new AWS.SNS({region: config.region});
};

module.exports.addDevice = function (data, callback) {
    utilities.logger.info('framework/notify/bin/awssns addDevice');
    sns.createPlatformEndpoint(data, function(e, result) {
        if (e) { utilities.logger.error(e); return callback ( new utilities.error.aws (e.code) ); }
        return callback (null, result.EndpointArn);
    });
};

module.exports.removeDevice = function (data, callback) {
    utilities.logger.info('framework/notify/bin/awssns removeDevice');
    sns.deleteEndpoint(data, function(e, result) {
        if (e) { utilities.logger.error(e); if (callback) return callback ( new utilities.error.aws (e.code) ); }
        if (callback) return callback (null, result);
    });
};


module.exports.setEndpointAttributes = function (data, callback) {
    utilities.logger.info('framework/notify/bin/awssns setEndpointAttributes');
    sns.setEndpointAttributes ( data, function(e, result) {
        if (e) { utilities.logger.error(e); return callback ( new utilities.error.aws (e.code) ); }
        return callback (null, result);
    });
};


module.exports.getEndpointAttributes = function (data, callback) {
    utilities.logger.info('framework/notify/bin/awssns getEndpointAttributes');
    sns.getEndpointAttributes(data, function(e, attributes) {
        if (e) { utilities.logger.error(e); return callback ( new utilities.error.aws (e.code) ); }
        return callback (null, attributes);
    });
};

if (process.env.testing === 'true') {    
    module.exports.sendNotification = function (data, callback) {
        utilities.logger.info('framework/notify/bin/awssns sendNotification');
        sns.publish (data, function (e, result){
            if (e) {
                utilities.logger.error(e);
                var err = new utilities.error.aws (e.code);
                if (err.code === 824) module.exports.removeDevice ({ EndpointArn: data.TargetArn });
            } else return callback(null, result);
        });
    }

} else {
    module.exports.sendNotification = function (data, callback) {
        utilities.logger.info('framework/notify/bin/awssns sendNotification');
        sns.publish (data, function (e, result){
            if (e) {
                utilities.logger.error(e);
                var err = new utilities.error.aws (e.code);
                if (err.code === 824) module.exports.removeDevice ( { EndpointArn: data.TargetArn } );
                return callback (err);
            }
        });
    }

}