var _               =   require('underscore'),
    AWS             =   require('aws-sdk'),
    sns             =   null,
    utilities       =   require('../utilities/index'),
    config          =   require('./config'),
    mailer          =   require('../mailer/index'),
    
    models          =   require ('mongoose').models,

    paramReplace    =   {
                            param1: function (str, data) { return str.replace (/param1/g, data); },
                            param2: function (str, data) { return str.replace (/param2/g, data); },
                            param3: function (str, data) { return str.replace (/param3/g, data); },
                            param4: function (str, data) { return str.replace (/param4/g, data); },
                            param5: function (str, data) { return str.replace (/param5/g, data); },
                        },
    test            =   'TEST',
    services        =   {};


function buildAWSSNSMessage (data) {
    if (!data.badge) data.badge = 0;
    
    var notification = config.notifications[data.id],
        message = { default: notification.message },
        customData = ",\\\"id\\\":\\\"" + data.id + "\\\"";
    
    if (data.custom && _.isObject(data.custom)) _.each (data.custom, function(prop, propName){ customData += ",\\\"" + propName + "\\\":\\\"" + prop + "\\\""; });
    
    //message [config.services.awssns.apnsId] = "{\"aps\":{\"alert\": \"" +  "Yo Frank! " +  "\",\"badge\": " +  "5" + ",\"sound\": \"" +  "bingbong.aiff" + "\",\"url\":\"www.amazon.com\"} }"
    
    message [config.services.awssns.apnsId] = "{\"aps\":{\"alert\": \"" +  notification.message +  "\",\"badge\": " +  data.badge + ",\"sound\": \"" +  (notification.sound || 'bingbong.aiff') + "\"}apnsCustom }";    
    
    message = (JSON.stringify(message)).replace(/apnsCustom/g, customData);

    if (data.msgParams) _.each ( _.pick (data.msgParams, 'param1', 'param2', 'param3', 'param4', 'param5'), function(parameter, paramName) { message = paramReplace [paramName] (message, parameter); }, this);
    
    return message;
};


function getSettings (person, product, callback) {
    if (!person.settings) {
        models.Setting.findOne()
            .where('_person').equals(person._id)
            .where('product').equals(product || 'shufl')
            .select('-_person')
            .read('secondaryPreferred')
            .lean().exec( function (e, setting) {  
                if (e) {
                    utilities.logger.error(e);
                    return callback ( null );
                } else return callback (setting); 
            });
    } else return callback (person.settings);    
}


//// Public --------------------------------------------------------------------------------------

module.exports.init = function (options)  { 
    if (options && _.isObject(options)) _.extend(config, options);
    // Set up enabled services
    _.each (config.services, function (service, serviceName){
        if (service.enabled) {
            services[serviceName] = require('./bin/' + serviceName);
            services[serviceName].init (service);
        }
        
    });
};

module.exports.addDevice = function (data, callback) { //data = {token: String, _id: String, platform: String}
    utilities.logger.info('framework/notify addDevice');
    if (services.awssns) {
        var params = { PlatformApplicationArn: config.services.awssns.PlatformApplicationArn, Token: data.token };
        services.awssns.addDevice(params, callback);
    } else return callback (null, null);
    
};

module.exports.removeDevice = function (data, callback) {
    utilities.logger.info('framework/notify removeDevice');

    if (services.awssns) { 
        var params = { EndpointArn: data.endpointArn };
        services.awssns.removeDevice (params, callback ? callback : null );
    } else return callback (null, null);
    
};

module.exports.enableDevice = function (data, callback) {
    utilities.logger.info('framework/notify enableDevice');
    
    if (services.awssns) { 
        var params = { EndpointArn: data.endpointArn, Attributes: { Enabled:'true' } };
        services.awssns.setEndpointAttributes (params, callback);
    } else return callback (null, null);
    
};

module.exports.disableDevice = function (data, callback) {
    utilities.logger.info('framework/notify disableDevice');
    
    if (services.awssns) { 
        var params = { EndpointArn: data.endpointArn, Attributes: { Enabled:'false' } };
        services.awssns.setEndpointAttributes (params, callback);
    } else return callback (null, null);
    
};

module.exports.setDeviceToken = function (data, callback) {
    utilities.logger.info('framework/notify setDeviceToken');
    
    if (services.awssns) { 
        var params = { EndpointArn: data.endpointArn, Attributes: { Enabled: 'true', Token: data.token } };
        services.awssns.setEndpointAttributes (params, callback);
    } else return callback (null, null);
    
};

module.exports.getDeviceAttributes = function (data, callback) {
    utilities.logger.info('framework/notify getDeviceAttributes');
    
    if (services.awssns) { 
        var params = { EndpointArn: data.endpointArn };
        services.awssns.getEndpointAttributes (params, callback);
    } else return callback (null, null);
    
};

if (process.env.testing === 'true') {    
    module.exports.sendNotification = function (data, callback) { //data = {person: Object, id: String, badge: number, custom: Object, msgParams: Object}
        utilities.logger.info('framework/notify sendNotification');
        if (!callback) callback = function(){};
        if (!services.awssns)              return callback (null, null);
        if (!data.person.notifications )        return callback (null, null);
        if (!data.person.notifications.length)  return callback (null, null);
        if (!config.notifications[data.id]) { utilities.logger.warn ('**** Notification type does not exist ***'); return callback (null, null); }
        
        if ( data.person.settings && data.person.settings.stopNotify && data.person.settings.stopNotify.length && _.indexOf(data.person.settings.stopNotify, data.id) > -1 ) return utilities.logger.info ( data.person.username || 'User' + ' has turned off ' + data.id + ' notification');
        var message = buildAWSSNSMessage(data);
        _.each (data.person.notifications, function(registration){
            if (registration.targetArn) services.awssns.sendNotification ({ TargetArn: registration.targetArn, Message: message, MessageStructure: 'json' }, callback);
            else return callback(null, null);
        });
        
    };
} else {
    module.exports.sendNotification = function (data) { //data = {person: Object, id: String, badge: number, custom: Object, msgParams: Object}
        utilities.logger.info('framework/notify sendNotification');

        if ( !config.notifications[data.id] ) return utilities.logger.warn ('**** Notification type does not exist ***');
        
        getSettings (data.person, data.product, function (settings) {
            
            if ( settings && settings.data && settings.data.stopNotifyEmail && settings.data.stopNotifyEmail.length && (_.indexOf(settings.data.stopNotifyEmail, data.id) > -1 || _.indexOf(settings.data.stopNotifyEmail, "all") > -1 || _.indexOf(settings.data.stopNotifyEmail, "disabled") > -1 ) )  utilities.logger.info ( (data.person.username || 'User') + ' has turned off ' + data.id + ' email notification');
            else {
                if (!data.person.email || data.person.email.split('@')[1] === 'facebook.com' || data.person.email.split('@')[1] === 'tfbnw.net') utilities.logger.warn ( (data.person.username || 'User') + ' does not have a proper email -  ' + data.person.email);
                
                var mailerArgs = {
                    to:         data.person.email,
                    product:    'shufl',
                    locale:     'en',
                    mailType:   data.id,
                    msgParams:  data.msgParams,
                    mailParams: data.mailParams
                };

                mailer.send ( mailerArgs, function(result) { });
            }

            if ( !services.awssns )                     return utilities.logger.warn ('AWSSNS service not initialized!');
            if ( !data.person.notifications )           return utilities.logger.info ('No notifications object found for user');
            if ( !data.person.notifications.length )    return utilities.logger.info ( data.person.username || 'User' + ' does not have notifications enabled');

            if ( settings &&  settings.data && settings.data.stopNotify && settings.data.stopNotify.length && (_.indexOf(settings.data.stopNotify, data.id) > -1 || _.indexOf(settings.data.stopNotify, "all") > -1 ) ) utilities.logger.info ( (data.person.username || 'User') + ' has turned off ' + data.id + ' notification');
            else {
                var message = buildAWSSNSMessage(data),
                    removeList = [],
                    newNotifications = [],
                    errorDone = _.after(data.person.notifications.length, function () {
                        if (removeList.length) {
                            newNotifications = _.reject (data.person.notifications, function (notification) { return _.indexOf(removeList, notification.targetArn) > -1 });
                            utilities.logger.info ("Removing " + removeList.length + " SNS disabled or invalid endpoint")
                            models.Person
                                .where({_id:data.person._id})
                                .update( { notifications: newNotifications }, function (e) {
                                    if (e) utilities.logger.error(e);
                                });
                        }
                    });

                _.each (data.person.notifications, function(registration){
                    if (registration.targetArn) services.awssns.sendNotification ({ TargetArn: registration.targetArn, Message: message, MessageStructure: 'json' }, function (e) {
                        if (e && ( e.code === 824 || e.code === 806 ) ) removeList.push (registration.targetArn);
                        errorDone();
                    });
                });        
            }   
        });
    };
}