var _               =   require('underscore'),
    AWS             =   require('aws-sdk'),
    csd             =   null,
    utilities       =   require('../utilities/index'),
    config          =   {},
    models          =   require ('mongoose').models,
    paramReplace    =   {
                            param1: function (str, data) { return str.replace (/param1/g, data); },
                            param2: function (str, data) { return str.replace (/param2/g, data); },
                            param3: function (str, data) { return str.replace (/param3/g, data); },
                            param4: function (str, data) { return str.replace (/param4/g, data); },
                            param5: function (str, data) { return str.replace (/param5/g, data); },
                        },
    services        =   {};


//// Public --------------------------------------------------------------------------------------

module.exports.init = function (options)  { 
    if (options && _.isObject(options)) _.extend(config, options);
    // Set up enabled services
    _.each (config.services, function (service, serviceName){
        if (service.enabled) {
            services[serviceName] = require('./bin/' + serviceName);
            services[serviceName].init (service);
        }
    });
};

module.exports.uploadDocuments = function (data, callback) { //data = {domainName: String, documents: Araay of documents}
    utilities.logger.info('framework/search uploadDocuments');
    if (services.awscs) {
        var domainSettings = config.services.awscs.domains[data.domainName],
            params = {
                domainName: data.domainName,
                params: {
                    contentType:    config.services.awscs.domains[data.domainName].contentType,
                    documents:      JSON.stringify ( domainSettings.itemAddFormat ? domainSettings.itemAddFormat (data.documents) : data.documents )
                }
            };
        services.awscs.uploadDocuments(params, callback);
    } else return callback (null, null);
};

module.exports.removeDocuments = function (data, callback) { //data = {domainName: String, documents: Araay of documents}
    utilities.logger.info('framework/search removeDocuments');
    if (services.awscs) {
        var domainSettings = config.services.awscs.domains[data.domainName],
            params = {
                domainName: data.domainName,
                params: {
                    contentType:    config.services.awscs.domains[data.domainName].contentType,
                    documents:      JSON.stringify ( domainSettings.itemRemFormat ? domainSettings.itemRemFormat (data.documents) : data.documents )
                }
            };
        services.awscs.uploadDocuments(params, callback);
    } else return callback (null, null);
};

module.exports.search = function (data, callback) { // data = {domainName: String, query: query String}
    utilities.logger.info('framework/search search');
    if ( services.awscs ) {
        var domainSettings = config.services.awscs.domains[data.domainName],
            params = {
                domainName: data.domainName,
                params: {
                    query:          domainSettings.queryFormat ? domainSettings.queryFormat(data) : "",
                    queryParser:    'structured',
                    size:           data.limit,
                    start:          data.skip || 0
                    //'return':       "itemid",
                }
            };

        if ( _.has (data, 'coords') ) {
            var expr = domainSettings.exprFormat ? domainSettings.exprFormat(data) : "";
            if (expr.length) params.params.expr = expr;
        }

        if (_.has (data, 'searchSort')) {
            var sort = domainSettings.sortFormat ? domainSettings.sortFormat(data) : "";
            if (sort.length) params.params.sort = sort;
        }

        utilities.logger.info(' ---- Search Parameters ---- ');
        utilities.logger.info(params);
        utilities.logger.info(' ---- ----------------- ---- ');

        services.awscs.search(params, callback);
    } else return callback (null, null);
};
