var underScore      =   require('underscore'),
    AWS             =   require('aws-sdk'),
    fs              =   require ('fs'),
    im              =   require('imagemagick'),
    s3              =   null,
    crypto          =   require('../crypto/index'),
    utilities       =   require('../utilities/index'),
    config          =   {},
    services        =   {};

function reverse (s) {
    var o = '';
    for (var i = s.length - 1; i >= 0; i--) { o += s[i] };
    return o;
}

function getUniqueKey ( id ) {
    var preHash = Date.now() + id,
        folder = reverse (id) + '/',
        filename =  crypto.createHashFilename(preHash);
    return folder + ( (filename && filename.hash) ? filename.hash : preHash );
}

function imageResize (data, callback) {
    if ( config.sizes[data.itemType] ) im.resize( { srcPath: data.localFile, dstPath: data.localFile, width: config.sizes[data.itemType].width }, function (e, stdout, stderr){
            if (e) { utilities.logger.error(e); return callback ( new utilities.error.app(436) ); }
            utilities.logger.info('++++ resized image ++++');
            return callback (null);
        });
    else return callback (null);
}

function deleteLocalFile (fullPath) { 
    fs.unlink (fullPath, function (e) { 
        if (e) {
            utilities.logger.error(e); 
            new utilities.error.srv(203); 
        } 
        else utilities.logger.info ('Deleted: ' + fullPath); 
    }); 
}

//// Public --------------------------------------------------------------------------------------

module.exports.init = function (options)  { 
    if (options && underScore.isObject(options)) underScore.extend(config, options);
    // Set up enabled services
    underScore.each (config.services, function (service, serviceName){
        
        if (service.enabled) {
            services[serviceName] = require('./bin/' + serviceName);
            services[serviceName].init (service);
        }

    });
};

module.exports.store = function (data, callback) { //data = { itemType: String, localFile: String, _id: String, region: String }
    utilities.logger.info('framework/storage store');
    if (services.awss3) {
        imageResize( data, function (e){
            if (e) return callback ( e );
            var item = config.services.awss3.itemTypes[data.itemType],
                params = {
                    localFile: data.localFile,
                    s3Params: { Bucket: item.bucket, Key: getUniqueKey(data._id), ACL: item.acl },
                    // other options supported by putObject, except Body and ContentLength. 
                    // See: http://docs.aws.amazon.com/AWSJavaScriptSDK/latest/AWS/S3.html#putObject-property 
                };
            
            services.awss3.store ( params, function (e) {  deleteLocalFile (data.localFile); });
            
            // return without waiting for upload to complete, assuming everything went ok. This is because s3 can be slow as molasses sometimes and we want to avoid a 504 timeout error.
            return callback (null, { 
                vendor: { id: 'awss3', bucket: params.s3Params.Bucket, key: params.s3Params.Key }, 
                url: (item.secure ? 'https://' : 'http://') +  params.s3Params.Bucket + '.s3.amazonaws.com/' + params.s3Params.Key
            });

        });
    } else return callback (null, null);
};

module.exports.storeSync = function (data, callback) { //data = { itemType: String, localFile: String, _id: String, region: String }
    utilities.logger.info('framework/storage storeSync');
    if (services.awss3) {
        imageResize( data, function (e){
            if (e) return callback ( e );
            var item = config.services.awss3.itemTypes[data.itemType],
                params = {
                    localFile: data.localFile,
                    s3Params: { Bucket: item.bucket, Key: getUniqueKey(data._id), ACL: item.acl },
                    // other options supported by putObject, except Body and ContentLength. 
                    // See: http://docs.aws.amazon.com/AWSJavaScriptSDK/latest/AWS/S3.html#putObject-property 
                };

            services.awss3.store ( params, function (e) { 
                if (e) { utilities.logger.error(e); return callback (e); }
                deleteLocalFile (data.localFile); 
                return callback (null, { 
                    vendor: { id: 'awss3', bucket: params.s3Params.Bucket, key: params.s3Params.Key }, 
                    url: item.secure ? services.awss3.getPublicLinkSecure (params.s3Params.Bucket, params.s3Params.Key ) : services.awss3.getPublicLink (params.s3Params.Bucket, params.s3Params.Key ) 
                });
            });
        });
    } else return callback (null, null);
};

module.exports.delete = function (data, callback) { //data = { id: String, bucket:String, key: String }
    utilities.logger.info('framework/storage delete');
    callback || ( callback = function(){} );
    
    if ( services.awss3 && data.id === 'awss3' ) {
        
        var item = config.services.awss3.itemTypes [data.itemType],
            params = {
                Bucket: data.bucket,
                Delete: {  Objects: [ { Key: data.key } ], }
            };

        services.awss3.delete ( params, function (e) { return callback (e); });

    } else return callback ( 1 );
};
