var _ 				= 	require('underscore'),
	models          =   require('mongoose').models,
    logger          =   require('../logger/index'),
    error           =   require('../error/index'),
    http            =   require('./bin/http'),
    transact        =   require('./bin/transact'),
    database        =   require('./bin/database'),
    escapeRegEx     =   {
                            allSpecial:                 /[-[\]{}()*+?.,"'&!:~\\^$|#\s]/g,
                            blackSlashAndApostrophe:    /['\\]/g
                        };



function getStuffAndCounts (data, callback) { // data -> {queryArgs, model, skip, propName, modelName}
    
    var error =             null,
        stuffReturn =       {},
        done =              function () {
                                return callback (error, stuffReturn);
                            };

    if (!data.skip ) {
        
        done = _.after (2, done);

        models[data.modelName]
            .where( data.queryArgs )
            .count( function (e, count ) {
                if (e) { 
                    logger.error(e); 
                    error = new error.db (102);
                }
                stuffReturn.count = count;
                done();
            });
    }

    data.model.exec(function (e, stuff){
        if (e) { 
            logger.error(e); 
            error = new error.db (102);
        }
        stuffReturn[data.propName] = stuff;
        done();
    });

};

function getStuffAndCountsEarlierThan (data, callback) { // data -> {queryArgs, model, earlierThan, propName, modelName}
    
    var error =             null,
        stuffReturn =       {},
        done =              function () {
                                return callback (error, stuffReturn);
                            };

    if (!data.earlierThan ) {
        
        done = _.after (2, done);

        models[data.modelName]
            .where( data.queryArgs )
            .count( function (e, count ) {
                if (e) { 
                    logger.error(e); 
                    error = new error.db (102);
                }
                stuffReturn.count = count;
                done();
            });
    } else {
        _.each (data.earlierThan, function (value, key){
            data.model.where(key).lt(value);    
        });
    }
    
    data.model.exec(function (e, stuff){
        if (e) { 
            logger.error(e); 
            error = new error.db (102);
        }
        stuffReturn[data.propName] = stuff;
        done();
    });

};

function escape (text) {
  if (_.isString(text)) return text.replace(escapeRegEx.allSpecial, "\\$&");
  else return text;
};

function escapeAWSCS (text) {
  if (_.isString(text)) return text.replace(escapeRegEx.allSpecial, "\\$&");
  else return text;
};

//// Public ---------------------------------------------------------------------------------------

exports.logger =   		      logger;
exports.error =   		      error;
exports.escape =              escape;
exports.escapeAWSCS =         escapeAWSCS;
exports.http =   		      http;
exports.transact =   	      transact;
exports.database =   	      database;
exports.getStuffAndCounts =   getStuffAndCounts;
exports.getStuffAndCountsEarlierThan =   getStuffAndCountsEarlierThan;