var underScore      =   require('underscore'),
    sanitizer       =   require('sanitizer'),
    utilities       =   require('../utilities/index'),
    config          =   {}, //require('./config'),
    tests           =   {},
    validationTest  =   {};


// return regex validator function
function createValidationTest (criteria) { return function (value) {  return config.regex[criteria].test (value); }; }

// return string matching function
function createStringMatchTest (criteria) { return function (value) {  return underScore.indexOf(config.stringSets[criteria], value) > -1 ? true : false ; }; }

// return object validator function
function createObjectValidationTest (criteria) { 
    var returnFunction = [ '(function (value) {' ,
        'if ( !underScore.isObject(value) ) return false;',
        'if ( exports.array (value) ) return false;',
        'if ( exports.string (value) ) return false;',
        'return ('
    ];
    
    underScore.each(config.objects[criteria], function(value, key, list){
        if (value[0].required) returnFunction.push( 'exports[\'' + value[0].test + '\'](value.' + key + ') &&' );
        else returnFunction.push( '( underScore.isUndefined (value.' + key + ') ? true : exports[\'' + value[0].test + '\'](value.' + key + ') ) &&' );
    });

    returnFunction.push (' true ) })');
    return eval(returnFunction.join(''));
}

// return function that checks an array of 'criteria' if each element is valid
function createValidationTestOnArray (criteria) {
    if ( !criteria || !exports[criteria] ) return function () { utilities.logger.error (criteria + ': no such validation function')};
    return function (value) {
        if ( !exports.array (value) )       return false;
        if ( !exports.notEmpty (value) )    return false;
        return underScore.every(value, exports[criteria]);
     };
}


//// Public --------------------------------------------------------------------------------------

exports.init = function (options)  { 
    if (options && underScore.isObject(options)) underScore.extend(config, options);

    //build list of string matchers based on String match returned function
    underScore.each (config.stringSets, function(value, key) { exports[key] = createStringMatchTest (key); });

    //build list of regex validators based on regex validator returned function
    underScore.each (config.regex,      function(value, key) { exports[key] = createValidationTest (key); });

    //build list of function validators
    underScore.each (config.functions,  function(value, key) { exports[key] = value; });

    //build list of object validators
    underScore.each (config.objects,    function(value, key) { exports[key + 'Object'] = createObjectValidationTest (key); });

    //build list of array validators
    underScore.each (config.arrays,     function(element, index) { exports[element + 'Array'] = createValidationTestOnArray (element); });
};

exports.missingProperties = function (obj, propsArray)  { 
    var missing = underScore.reject(propsArray, function(prop){ return underScore.has(obj, prop); });
    if (missing.length) utilities.logger.error ('Missing properties: ' + missing );
    return missing.length; 
};

exports.isValid = function (data, properties) {
    if ( underScore.isUndefined(data) || underScore.isNull(data) || underScore.isFunction(data) || !underScore.isObject(data) || underScore.isEmpty(data) ) return false;
    if ( underScore.isUndefined(properties) || underScore.isNull(properties) || underScore.isFunction(properties) || !underScore.isArray(properties) || underScore.isEmpty(properties) ) return false;
    var notValid = underScore.find (properties, function(property){
        if (!config.rules[property]) { 
            utilities.logger.error ('Valid rules do not exist for ' + property + ' property!'); return true; 
        }
        var notValidAgain = underScore.find (config.rules[property], function(rule){
            var ruleTest = !exports[rule]( data[property] );
            if (ruleTest) { 
                utilities.logger.error (property + ' property did not pass the ' + rule + ' test.'); 
                if (property !== 'password') utilities.logger.error (data); 
            }
            return ruleTest;
        });
        return !underScore.isUndefined(notValidAgain);
    });
    return !notValid;
};

exports.sanitize = function (data, callback) {
    var sanitizeObject = function () {
        underScore.each (data, function (element, index) { exports.sanitize(element, function (result) { data[index] = result; } ); });
        return callback(data);
    };
    
    if (data.email) data.email = data.email.toLowerCase(); // change emails to lowercase

    if (underScore.isString(data)) return callback(sanitizer.unescapeEntities(sanitizer.sanitize(data)));
    else if (underScore.isFunction(data)) return callback(null);
    else if (underScore.isObject(data)) sanitizeObject ();
    else return callback(data);
};